
#include "Pinger.hh"

#include "PingPongProtocol.hh"
#include "umlrtcommsportrole.hh"
#include "umlrtmessage.hh"
#include "umlrtslot.hh"
#include <cstddef>
#include "umlrtcapsuleclass.hh"
#include "umlrtframeservice.hh"
class UMLRTRtsInterface;
struct UMLRTCommsPort;

#include "utility_macros.hh"

Capsule_Pinger::Capsule_Pinger( const UMLRTCapsuleClass * cd, UMLRTSlot * st, const UMLRTCommsPort * * border, const UMLRTCommsPort * * internal, bool isStat )
: UMLRTCapsule( NULL, cd, st, border, internal, isStat )
, pingPort( borderPorts[borderport_pingPort] )
, currentState( SPECIAL_INTERNAL_STATE_UNVISITED )
{
    stateNames[Playing] = "Playing";
    stateNames[SPECIAL_INTERNAL_STATE_UNVISITED] = "<uninitialized>";
}






void Capsule_Pinger::bindPort( bool isBorder, int portId, int index )
{
    if( isBorder )
        switch( portId )
        {
        case borderport_log:
            UMLRTFrameService::sendBoundUnbound( borderPorts, borderport_log, index, true );
            break;
        case borderport_pingPort:
            UMLRTFrameService::sendBoundUnbound( borderPorts, borderport_pingPort, index, true );
            break;
        }
}

void Capsule_Pinger::unbindPort( bool isBorder, int portId, int index )
{
    if( isBorder )
        switch( portId )
        {
        case borderport_log:
            UMLRTFrameService::sendBoundUnbound( borderPorts, borderport_log, index, false );
            UMLRTFrameService::disconnectPort( borderPorts[borderport_log], index );
            break;
        case borderport_pingPort:
            UMLRTFrameService::sendBoundUnbound( borderPorts, borderport_pingPort, index, false );
            UMLRTFrameService::disconnectPort( borderPorts[borderport_pingPort], index );
            break;
        }
}

void Capsule_Pinger::inject( const UMLRTMessage & message )
{
    msg = &message;
    switch( currentState )
    {
    case Playing:
        currentState = state_____Playing( &message );
        break;
    default:
        break;
    }
}

void Capsule_Pinger::initialize( const UMLRTMessage & message )
{
    msg = &message;
    actionchain_____Initial( &message );
    currentState = Playing;
}

const char * Capsule_Pinger::getCurrentStateString() const
{
    return stateNames[currentState];
}




void Capsule_Pinger::entryaction_____Playing( const UMLRTMessage * msg )
{
    #define rtdata ( (void *)msg->getParam( 0 ) )
    /* UMLRTGEN-USERREGION-BEGIN platform:/resource/PingPong/PingPong.uml PingPong::Pinger::Playing entry  */
    log.log(LCAPINST, "(enter Playing) sending ping");
    pingPort.ping().send();
    /* UMLRTGEN-USERREGION-END */
    #undef rtdata
}

void Capsule_Pinger::exitaction_____Playing( const UMLRTMessage * msg )
{
    #define rtdata ( (void *)msg->getParam( 0 ) )
    /* UMLRTGEN-USERREGION-BEGIN platform:/resource/PingPong/PingPong.uml PingPong::Pinger::Playing exit  */
    /* UMLRTGEN-USERREGION-END */
    #undef rtdata
}

void Capsule_Pinger::transitionaction_____onPong( const UMLRTMessage * msg )
{
    #define rtdata ( (void *)msg->getParam( 0 ) )
    /* UMLRTGEN-USERREGION-BEGIN platform:/resource/PingPong/PingPong.uml PingPong::Pinger transition Playing,Playing,pong:pingPort */
    log.log(LCAPINST, "(onPong)");
    /* UMLRTGEN-USERREGION-END */
    #undef rtdata
}

void Capsule_Pinger::actionchain_____Initial( const UMLRTMessage * msg )
{
    entryaction_____Playing( msg );
}

void Capsule_Pinger::actionchain_____onPong( const UMLRTMessage * msg )
{
    exitaction_____Playing( msg );
    transitionaction_____onPong( msg );
    entryaction_____Playing( msg );
}

Capsule_Pinger::State Capsule_Pinger::state_____Playing( const UMLRTMessage * msg )
{
    switch( msg->destPort->role()->id )
    {
    case port_pingPort:
        switch( msg->getSignalId() )
        {
        case PingPongProtocol::signal_pong:
            actionchain_____onPong( msg );
            return Playing;
        default:
            this->unexpectedMessage();
            break;
        }
        return currentState;
    default:
        this->unexpectedMessage();
        break;
    }
    return currentState;
}


static const UMLRTCommsPortRole portroles_border[] = 
{
    {
        Capsule_Pinger::port_pingPort,
        "PingPongProtocol",
        "pingPort",
        "",
        1,
        true,
        false,
        false,
        false,
        false,
        false,
        true
    },
    {
        Capsule_Pinger::port_log,
        "Log",
        "log",
        "",
        1,
        true,
        false,
        false,
        false,
        false,
        false,
        true
    }
};

static void instantiate_Pinger( const UMLRTRtsInterface * rts, UMLRTSlot * slot, const UMLRTCommsPort * * borderPorts )
{
    slot->capsule = new Capsule_Pinger( &Pinger, slot, borderPorts, NULL, false );
}

const UMLRTCapsuleClass Pinger = 
{
    "Pinger",
    NULL,
    instantiate_Pinger,
    0,
    NULL,
    2,
    portroles_border,
    0,
    NULL
};

