package org.eclipse.papyrusrt.xtumlrt.trans.to.uml.umlrt;

import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.transaction.RecordingCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gmf.runtime.notation.EObjectListValueStyle;
import org.eclipse.gmf.runtime.notation.NotationPackage;
import org.eclipse.papyrus.commands.Activator;
import org.eclipse.papyrus.infra.core.editor.BackboneException;
import org.eclipse.papyrus.infra.core.resource.ModelMultiException;
import org.eclipse.papyrus.infra.core.resource.ModelSet;
import org.eclipse.papyrus.infra.core.resource.NotFoundException;
import org.eclipse.papyrus.infra.core.sashwindows.di.service.IPageManager;
import org.eclipse.papyrus.infra.core.services.ExtensionServicesRegistry;
import org.eclipse.papyrus.infra.core.services.ServiceException;
import org.eclipse.papyrus.infra.core.services.ServicesRegistry;
import org.eclipse.papyrus.infra.emf.utils.ServiceUtilsForEObject;
import org.eclipse.papyrus.infra.gmfdiag.css.notation.CSSDiagram;
import org.eclipse.papyrus.infra.gmfdiag.css.resource.CSSNotationModel;
import org.eclipse.papyrus.infra.gmfdiag.css.stylesheets.EmbeddedStyleSheet;
import org.eclipse.papyrus.infra.gmfdiag.css.stylesheets.StylesheetsFactory;
import org.eclipse.papyrus.infra.ui.extension.commands.IModelCreationCommand;
import org.eclipse.papyrus.infra.viewpoints.configuration.PapyrusDiagram;
import org.eclipse.papyrus.infra.viewpoints.configuration.PapyrusView;
import org.eclipse.papyrus.infra.viewpoints.policy.PolicyChecker;
import org.eclipse.papyrus.infra.viewpoints.policy.ViewPrototype;
import org.eclipse.papyrus.uml.diagram.sequence.CreateSequenceDiagramCommand;
import org.eclipse.papyrus.uml.diagram.wizards.category.DiagramCategoryRegistry;
import org.eclipse.papyrus.uml.tools.model.UmlModel;
import org.eclipse.papyrus.uml.tools.utils.PackageUtil;
import org.eclipse.papyrusrt.codegen.cpp.AnsiCLibraryMetadata;
import org.eclipse.papyrusrt.codegen.cpp.profile.facade.RTCppPropertiesProfileMetadata;
import org.eclipse.papyrusrt.xtumlrt.external.ExternalPackageManager;
import org.eclipse.papyrusrt.xtumlrt.external.ExternalPackageMetadata;
import org.eclipse.papyrusrt.xtumlrt.util.XTUMLRTLogger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.util.UMLUtil;

/**
 * XTUMLRT to UML translator utility class.
 * 
 * @author ysroh
 *
 */
public final class Xtumlrt2UMLTranslatorUtils {
	/**
	 * 
	 */
	public static final String SM_PROFILE_PATH = "pathmap://UML_RT_PROFILE/UMLRealTimeSM-addendum.profile.uml"; //$NON-NLS-1$
	/**
	 * 
	 */
	public static final String CPP_PROFILE_PATH = "pathmap://UMLRT_CPP/RTCppProperties.profile.uml"; //$NON-NLS-1$
	/**
	 * 
	 */
	public static final String ANSI_C_LIB_ID = "org.eclipse.papyrus.designer.languages.cpp.library";
	/**
	 * 
	 */
	public static final String ANSI_C_LIB_PATH = "pathmap://PapyrusC_Cpp_LIBRARIES/AnsiCLibrary.uml";
	/**
	 * 
	 */
	public static final String RTS_MODLIB_ID = "org.eclipse.papyrusrt.umlrt.common.rts.library";
	/**
	 * 
	 */
	public static final String RTS_MODLIB_PATH = "pathmap://UMLRTRTSLIB/UMLRT-RTS.uml";
	/**
	 * 
	 */
	public static final String DIAGRAM_CATEGORY_ID = "UMLRT";
	/**
	 * 
	 */
	public static final ResourceSetImpl RSET = new ResourceSetImpl();
	/**
	 * 
	 */
	private static Resource UMLPrimitiveTypeResource;
	/**
	 * 
	 */
	private static Resource AnsiCLibraryResource;
	/**
	 * 
	 */
	private static Resource RTSLibraryResource;

	/**
	 * 
	 */
	private static boolean initialized = false;

	/**
	 * 
	 */
	private static final ExternalPackageManager EXT_PKG_MGR = ExternalPackageManager.getInstance();

	/** The collection of 'known' packages that must be loaded and registered. */
	private static final ExternalPackageMetadata[] REQUIRED_PACKAGES = {
			RTCppPropertiesProfileMetadata.INSTANCE,
			AnsiCLibraryMetadata.INSTANCE
	};


	/**
	 * ToDo: need to be refactored.
	 */
	public static void initializeRequiredLibraries() {
		if (initialized) {
			return;
		}
		for (ExternalPackageMetadata metadata : REQUIRED_PACKAGES) {
			EXT_PKG_MGR.addRequiredPackage(metadata);
		}
		EXT_PKG_MGR.setup();
		UMLPrimitiveTypeResource = EXT_PKG_MGR.getResourceSet()
				.getResource(URI.createURI("pathmap://UML_LIBRARIES/UMLPrimitiveTypes.library.uml"), true);
		String ansiCLibPath = ANSI_C_LIB_PATH;
		String rtsLibPath = RTS_MODLIB_PATH;
		if (EXT_PKG_MGR != null) {
			Map<String, ExternalPackageMetadata> registry = EXT_PKG_MGR.getRegistry();
			if (registry != null) {
				ExternalPackageMetadata ansiCmetadata = registry.get(ANSI_C_LIB_ID);
				if (ansiCmetadata != null) {
					String pathmap = ansiCmetadata.getPathmap();
					if (pathmap != null && !pathmap.isEmpty()) {
						ansiCLibPath = pathmap;
					}
				}
				ExternalPackageMetadata rtsLibMetadata = registry.get(RTS_MODLIB_ID);
				if (rtsLibMetadata != null) {
					String pathmap = rtsLibMetadata.getPathmap();
					if (pathmap != null && !pathmap.isEmpty()) {
						rtsLibPath = pathmap;
					}
				}
			}
		}
		AnsiCLibraryResource = RSET.getResource(URI.createURI(ansiCLibPath), true);
		RTSLibraryResource = RSET.getResource(URI.createURI(rtsLibPath), true);
		initialized = true;
	}

	/**
	 * Constructor.
	 *
	 */
	private Xtumlrt2UMLTranslatorUtils() {
		// TODO Auto-generated constructor stub
	}

	/**
	 * Ask user confirmation to overwrite existing model.
	 * 
	 * @return OK or CANCEL
	 * @throws ServiceException
	 * @throws BackboneException
	 * @throws IOException
	 * @throws ModelMultiException
	 */
	public static int overwrite() {
		// overwrite message
		MessageBox dialog = new MessageBox(Display.getCurrent().getActiveShell(), SWT.ICON_WARNING | SWT.OK | SWT.CANCEL);
		dialog.setText("Warning");
		dialog.setMessage("Do you want to overwrite the existing model?");

		int result = dialog.open();
		if (result == SWT.OK) {
			return Status.OK;
		}
		return Status.CANCEL;

	}

	/**
	 * Check if model already exist.
	 * 
	 * @param uri
	 *            model uri
	 * @return true if exist. false otherwise
	 */
	public static boolean modelExists(final URI uri) {
		IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(uri.toPlatformString(true)));
		return file.exists();
	}

	/**
	 * Create empty Papyrus UML model.
	 * 
	 * @param registry
	 *            service registry
	 * @param uri
	 *            URI of the papyrus model to create
	 * @return model set
	 * @throws ServiceException
	 * @throws IOException
	 * @throws BackboneException
	 * @throws ModelMultiException
	 */
	public static ModelSet createPapyrusModel(ServicesRegistry registry, final URI uri) throws ServiceException, BackboneException, IOException {
		final ModelSet modelSet = registry.getService(ModelSet.class);
		TransactionalEditingDomain domain = modelSet.getTransactionalEditingDomain();
		// create Papyrus model
		RecordingCommand command = new RecordingCommand(domain) {

			@Override
			protected void doExecute() {
				modelSet.createModels(uri);
			}
		};
		domain.getCommandStack().execute(command);

		initRegistry(registry);

		initDomainModel(modelSet);
		return modelSet;

	}

	/**
	 * Initialize model.
	 * 
	 * @param modelSet
	 *            model set
	 * @throws BackboneException
	 *             exception
	 * @throws IOException
	 *             exception
	 */
	public static void initDomainModel(final ModelSet modelSet) throws BackboneException, IOException {
		TransactionalEditingDomain domain = modelSet.getTransactionalEditingDomain();

		// create empty domain model for UML-RT
		IModelCreationCommand creationCommand;
		creationCommand = DiagramCategoryRegistry.getInstance().getDiagramCategoryMap().get(DIAGRAM_CATEGORY_ID)
				.getCommand();
		creationCommand.createModel(modelSet);

		// Apply state machine profile
		RecordingCommand smcommand = new RecordingCommand(domain) {

			@Override
			protected void doExecute() {
				Package model;
				try {
					model = (Package) ((UmlModel) modelSet.getModel(UmlModel.MODEL_ID)).lookupRoot();
					// Apply state machine profile
					Profile smProfile = (Profile) PackageUtil.loadPackage(URI.createURI(SM_PROFILE_PATH),
							model.eResource().getResourceSet());
					if (smProfile != null) {
						PackageUtil.applyProfile(model, smProfile, true);
					}
					// Apply RtCppProperties profile
					Profile cppProfile = (Profile) PackageUtil.loadPackage(URI.createURI(CPP_PROFILE_PATH),
							model.eResource().getResourceSet());
					if (smProfile != null) {
						PackageUtil.applyProfile(model, cppProfile, true);
					}
				} catch (NotFoundException e) {
					XTUMLRTLogger.error("Failed to initialize domain model", e);
				}
			}
		};
		domain.getCommandStack().execute(smcommand);
		modelSet.save(new NullProgressMonitor());
	}

	/**
	 * Start service registry.
	 * 
	 * @param registry
	 *            service registry
	 */
	public static void initRegistry(ServicesRegistry registry) {
		// init registry
		try {
			registry.startRegistry();
			registry.getService(IPageManager.class);
		} catch (ServiceException ex) {
			// Ignore this exception: some services may not have been
			// loaded, which is probably normal at this point
		}
	}

	/**
	 * Create service registry.
	 * 
	 * @return service registry
	 */
	public static ServicesRegistry createServicesRegistry() {
		ServicesRegistry registry = null;

		try {
			registry = new ExtensionServicesRegistry(org.eclipse.papyrus.infra.core.Activator.PLUGIN_ID);
			registry.startServicesByClassKeys(ModelSet.class);
		} catch (ServiceException e) {
			XTUMLRTLogger.error("Failed to create service registry", e);
		}
		return registry;
	}

	/**
	 * Apply stereotype.
	 * 
	 * @param element
	 *            element to apply stereotype
	 * @param definition
	 *            eclass
	 * @return stereotype
	 */
	public static EObject applyStereotype(Element element, EClass definition) {
		return applyStereotype(element, element, definition);
	}

	/**
	 * Apply stereotype with context.
	 * 
	 * @param element
	 *            element to apply stereotype
	 * @param context
	 *            context
	 * @param definition
	 *            eclass
	 * @return stereotype
	 */
	public static EObject applyStereotype(Element element, Element context, EClass definition) {
		return UMLUtil.StereotypeApplicationHelper.getInstance(context).applyStereotype(element, definition);
	}

	/**
	 * Queries primitive type mapping.
	 * 
	 * @param type
	 *            type name
	 * @return primitive type
	 */
	public static EObject getPrimitiveType(String type) {

		Collection<NamedElement> results = UMLUtil.findNamedElements(AnsiCLibraryResource, "AnsiCLibrary::" + type);
		if (!results.isEmpty()) {
			return results.iterator().next();
		}
		results = UMLUtil.findNamedElements(UMLPrimitiveTypeResource, "PrimitiveTypes::" + type);
		if (!results.isEmpty()) {
			return results.iterator().next();
		}

		return null;
	}

	/**
	 * Get RTS protocol from the library.
	 * 
	 * @param name
	 *            protocol name
	 * @return protocol
	 */
	public static EObject getRTSProtocol(String name) {
		Collection<NamedElement> results = UMLUtil.findNamedElements(RTSLibraryResource,
				"UMLRT-RTS::" + name + "::" + name);
		if (!results.isEmpty()) {
			return results.iterator().next();
		}
		return null;
	}

	/**
	 * Get modelSet from given EObject.
	 * 
	 * @param eobject
	 *            EObject
	 * @return ModelSet
	 */
	public static ModelSet getModelSetFromEObject(EObject eobject) {
		ModelSet result = null;
		ServicesRegistry registry;
		try {
			registry = ServiceUtilsForEObject.getInstance().getServiceRegistry(eobject);
		} catch (ServiceException ex) {
			Activator.log.error(ex);
			return result;
		}
		try {
			result = registry.getService(ModelSet.class);
		} catch (ServiceException ex) {
			Activator.log.error(ex);
			return result;
		}
		return result;
	}

	/**
	 * Get all interactions from given model.
	 * 
	 * @param model
	 *            Model
	 * @return set of interactions
	 */
	public static Set<Interaction> getAllInteractions(Model model) {
		Set<Interaction> interactions = new HashSet<>();
		TreeIterator<EObject> itor = model.eAllContents();
		while (itor.hasNext()) {
			EObject next = itor.next();
			if (next instanceof Interaction) {
				interactions.add((Interaction) next);
			}
			if (!(next instanceof Package) && !(next instanceof Class)) {
				itor.prune();
			}
		}
		return interactions;
	}

	/**
	 * Create sequence diagram from given Interaction.
	 * 
	 * @param model
	 *            interaction
	 */
	@SuppressWarnings("unchecked")
	public static void createSequenceDiagrams(Model model) {

		Set<Interaction> interactions = getAllInteractions(model);
		if (interactions.isEmpty()) {
			return;
		}

		ModelSet modelSet = getModelSetFromEObject(model);

		if (modelSet == null) {
			return;
		}

		ViewPrototype seqDiagPrototype = null;
		Collection<ViewPrototype> prototypes = PolicyChecker.getCurrent().getPrototypesFor(model);
		for (ViewPrototype prototype : prototypes) {
			PapyrusView view = prototype.getConfiguration();
			if (view instanceof PapyrusDiagram && "Sequence Diagram".equals(prototype.getLabel())) {
				seqDiagPrototype = prototype;
				break;
			}
		}

		if (seqDiagPrototype == null) {
			return;
		}

		CreateSequenceDiagramCommand cmd = new CreateSequenceDiagramCommand();

		EmbeddedStyleSheet es = StylesheetsFactory.eINSTANCE.createEmbeddedStyleSheet();
		es.setLabel("CanonicalDiagram");
		es.setContent("{canonical:true;}");

		CSSNotationModel notation = (CSSNotationModel) modelSet.getModel("org.eclipse.papyrus.infra.core.resource.notation.NotationModel");
		notation.getResource().getContents().add(es);

		for (Interaction i : interactions) {
			CSSDiagram di = (CSSDiagram) cmd.createDiagram(modelSet, i, i, seqDiagPrototype, "seqDiagram", false);
			if (di != null) {
				EObjectListValueStyle valuestyle = (EObjectListValueStyle) di.createStyle(NotationPackage.Literals.EOBJECT_LIST_VALUE_STYLE);
				valuestyle.setName("css_stylesheets");
				valuestyle.getEObjectListValue().add(es);
			}
		}
	}
}
