/*****************************************************************************
 * Copyright (c) 2014 CEA LIST.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  CEA LIST - Initial API and implementation
 *****************************************************************************/
package org.eclipse.papyrusrt.umlrt.core.tests.utils;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.eclipse.emf.common.command.Command;
import org.eclipse.papyrus.junit.utils.rules.PluginResource;
import org.eclipse.papyrusrt.umlrt.core.utils.RTPortKindEnum;
import org.eclipse.papyrusrt.umlrt.core.utils.RTPortUtils;
import org.eclipse.papyrusrt.umlrt.profile.UMLRealTime.PortRegistrationType;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.VisibilityKind;
import org.junit.Assert;
import org.junit.Test;


/**
 * Test class for {@link RTPortUtils}
 */
@PluginResource("resource/Utils.di")
public class RTPortUtilsTest extends AbstractRTPortTest {

	/**
	 * Test {@link RTPortUtils#getKind(Port)}
	 */
	@Test
	public void testRTPortUtils_ExternalKind() {
		assertEquals("error in RTPortUtils#getKind(Port)", RTPortKindEnum.EXTERNAL, RTPortUtils.getKind(capsule_externalBehavior));
	}

	/**
	 * Test {@link RTPortUtils#getKind(Port)}
	 */
	@Test
	public void testRTPortUtils_InternalKind() {
		assertEquals("error in RTPortUtils#getKind(Port)", RTPortKindEnum.INTERNAL, RTPortUtils.getKind(capsule_internalBehavior));
	}

	/**
	 * Test {@link RTPortUtils#getKind(Port)}
	 */
	@Test
	public void testRTPortUtils_RelayKind() {
		assertEquals("error in RTPortUtils#getKind(Port)", RTPortKindEnum.RELAY, RTPortUtils.getKind(capsule_relay));
	}

	/**
	 * Test {@link RTPortUtils#getKind(Port)}
	 */
	@Test
	public void testRTPortUtils_SAPKind() {
		assertEquals("error in RTPortUtils#getKind(Port)", RTPortKindEnum.SAP, RTPortUtils.getKind(capsule_sap));
	}

	/**
	 * Test {@link RTPortUtils#getKind(Port)}
	 */
	@Test
	public void testRTPortUtils_SPPKind() {
		assertEquals("error in RTPortUtils#getKind(Port)", RTPortKindEnum.SPP, RTPortUtils.getKind(capsule_spp));
	}

	/**
	 * Test {@link RTPortUtils#getKind(Port)}
	 */
	@Test
	public void testRTPortUtils_legacySAPKind() {
		assertEquals("error in RTPortUtils#getKind(Port)", RTPortKindEnum.SAP, RTPortUtils.getKind(capsule_legacySap));
	}

	/**
	 * Test {@link RTPortUtils#getKind(Port)}
	 */
	@Test
	public void testRTPortUtils_legacySPPKind() {
		assertEquals("error in RTPortUtils#getKind(Port)", RTPortKindEnum.SPP, RTPortUtils.getKind(capsule_legacySpp));
	}

	// ------------------------------------------------------------------------
	// RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)
	// ------------------------------------------------------------------------

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromSAPtoSPP() throws Exception {
		assertNotEquals("registration error in input model", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_sap));
		assertNotEquals("registration override error in input model", "", RTPortUtils.getRegistrationOverride(capsule_sap));
		testRTPortUtils_getChangeKindCommand(capsule_sap, RTPortKindEnum.SPP);
		assertNotEquals("Registration: error in RTPortUtils#getChangeKindCommand(Port)", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_sap));
		assertNotEquals("RegistrationOverride: error in RTPortUtils#getChangeKindCommand(Port)", "", RTPortUtils.getRegistrationOverride(capsule_sap));
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromSAPtoSAP() throws Exception {
		assertNotEquals("registration error in input model", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_sap));
		assertNotEquals("registration override error in input model", "", RTPortUtils.getRegistrationOverride(capsule_sap));
		testRTPortUtils_getChangeKindCommand(capsule_sap, RTPortKindEnum.SAP);
		assertNotEquals("Registration: error in RTPortUtils#getChangeKindCommand(Port)", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_sap));
		assertNotEquals("RegistrationOverride: error in RTPortUtils#getChangeKindCommand(Port)", "", RTPortUtils.getRegistrationOverride(capsule_sap));
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromSAPtoExternal() throws Exception {
		assertNotEquals("registration error in input model", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_sap));
		assertNotEquals("registration override error in input model", "", RTPortUtils.getRegistrationOverride(capsule_sap));
		testRTPortUtils_getChangeKindCommand(capsule_sap, RTPortKindEnum.EXTERNAL);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromSAPtoInternal() throws Exception {
		assertNotEquals("registration error in input model", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_sap));
		assertNotEquals("registration override error in input model", "", RTPortUtils.getRegistrationOverride(capsule_sap));
		testRTPortUtils_getChangeKindCommand(capsule_sap, RTPortKindEnum.INTERNAL);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromSAPtoRelay() throws Exception {
		assertNotEquals("registration error in input model", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_sap));
		assertNotEquals("registration override error in input model", "", RTPortUtils.getRegistrationOverride(capsule_sap));
		testRTPortUtils_getChangeKindCommand(capsule_sap, RTPortKindEnum.RELAY);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromSPPtoSPP() throws Exception {
		assertNotEquals("registration error in input model", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_spp));
		assertNotEquals("registration override error in input model", "", RTPortUtils.getRegistrationOverride(capsule_spp));
		testRTPortUtils_getChangeKindCommand(capsule_spp, RTPortKindEnum.SPP);
		assertNotEquals("Registration: error in RTPortUtils#getChangeKindCommand(Port)", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_spp));
		assertNotEquals("RegistrationOverride: error in RTPortUtils#getChangeKindCommand(Port)", "", RTPortUtils.getRegistrationOverride(capsule_spp));
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromSPPtoSAP() throws Exception {
		assertNotEquals("registration error in input model", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_spp));
		assertNotEquals("registration override error in input model", "", RTPortUtils.getRegistrationOverride(capsule_spp));
		testRTPortUtils_getChangeKindCommand(capsule_spp, RTPortKindEnum.SAP);
		assertNotEquals("Registration: error in RTPortUtils#getChangeKindCommand(Port)", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_spp));
		assertNotEquals("RegistrationOverride: error in RTPortUtils#getChangeKindCommand(Port)", "", RTPortUtils.getRegistrationOverride(capsule_spp));
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromSPPtoExternal() throws Exception {
		assertNotEquals("registration error in input model", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_spp));
		assertNotEquals("registration override error in input model", "", RTPortUtils.getRegistrationOverride(capsule_spp));
		testRTPortUtils_getChangeKindCommand(capsule_spp, RTPortKindEnum.EXTERNAL);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromSPPtoInternal() throws Exception {
		assertNotEquals("registration error in input model", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_spp));
		assertNotEquals("registration override error in input model", "", RTPortUtils.getRegistrationOverride(capsule_spp));
		testRTPortUtils_getChangeKindCommand(capsule_spp, RTPortKindEnum.INTERNAL);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromSPPtoRelay() throws Exception {
		assertNotEquals("registration error in input model", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(capsule_spp));
		assertNotEquals("registration override error in input model", "", RTPortUtils.getRegistrationOverride(capsule_spp));
		testRTPortUtils_getChangeKindCommand(capsule_spp, RTPortKindEnum.RELAY);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromRelaytoSPP() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_relay, RTPortKindEnum.SPP);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromRelaytoSAP() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_relay, RTPortKindEnum.SAP);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromRelaytoExternal() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_relay, RTPortKindEnum.EXTERNAL);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromRelaytoInternal() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_relay, RTPortKindEnum.INTERNAL);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromRelaytoRelay() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_relay, RTPortKindEnum.RELAY);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromExternaltoSPP() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_externalBehavior, RTPortKindEnum.SPP);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromExternaltoSAP() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_externalBehavior, RTPortKindEnum.SAP);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromExternaltoExternal() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_externalBehavior, RTPortKindEnum.EXTERNAL);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromExternaltoInternal() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_externalBehavior, RTPortKindEnum.INTERNAL);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromExternaltoRelay() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_externalBehavior, RTPortKindEnum.RELAY);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromInternaltoSPP() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_internalBehavior, RTPortKindEnum.SPP);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromInternaltoSAP() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_internalBehavior, RTPortKindEnum.SAP);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromInternaltoExternal() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_internalBehavior, RTPortKindEnum.EXTERNAL);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromInternaltoInternal() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_internalBehavior, RTPortKindEnum.INTERNAL);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeFromInternaltoRelay() throws Exception {
		testRTPortUtils_getChangeKindCommand(capsule_internalBehavior, RTPortKindEnum.RELAY);
	}


	// ------------------------------------------------------------------------
	// RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum, boolean)
	// ------------------------------------------------------------------------

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum, boolean)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromSAPtoSPP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_sap, RTPortKindEnum.SPP, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommandWithCheck(Port, RTPortKindEnum, boolean)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromSAPtoSAP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_sap, RTPortKindEnum.SAP, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommandWithCheck(Port, RTPortKindEnum, boolean)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromSAPtoExternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_sap, RTPortKindEnum.EXTERNAL, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromSAPtoInternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_sap, RTPortKindEnum.INTERNAL, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromSAPtoRelay() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_sap, RTPortKindEnum.RELAY, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromSPPtoSPP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_spp, RTPortKindEnum.SPP, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromSPPtoSAP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_spp, RTPortKindEnum.SAP, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromSPPtoExternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_spp, RTPortKindEnum.EXTERNAL, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromSPPtoInternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_spp, RTPortKindEnum.INTERNAL, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromSPPtoRelay() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_spp, RTPortKindEnum.RELAY, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelaytoSPP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_relay, RTPortKindEnum.SPP, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelaytoSAP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_relay, RTPortKindEnum.SAP, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelaytoExternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_relay, RTPortKindEnum.EXTERNAL, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelaytoInternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_relay, RTPortKindEnum.INTERNAL, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelaytoRelay() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_relay, RTPortKindEnum.RELAY, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternaltoSPP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_externalBehavior, RTPortKindEnum.SPP, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternaltoSAP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_externalBehavior, RTPortKindEnum.SAP, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternaltoExternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_externalBehavior, RTPortKindEnum.EXTERNAL, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternaltoInternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_externalBehavior, RTPortKindEnum.INTERNAL, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternaltoRelay() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_externalBehavior, RTPortKindEnum.RELAY, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternaltoSPP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_internalBehavior, RTPortKindEnum.SPP, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternaltoSAP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_internalBehavior, RTPortKindEnum.SAP, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternaltoExternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_internalBehavior, RTPortKindEnum.EXTERNAL, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternaltoInternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_internalBehavior, RTPortKindEnum.INTERNAL, true);
	}

	/**
	 * Test {@link RTPortUtils#getChangeKindCommand(Port, RTPortKindEnum)}
	 */
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternaltoRelay() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(capsule_internalBehavior, RTPortKindEnum.RELAY, true);
	}


	// check the specific ones with connection (internal, external and relay)
	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelayConnectedInsideToRelay() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_relay, RTPortKindEnum.RELAY, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelayConnectedInsideToExternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_relay, RTPortKindEnum.EXTERNAL, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelayConnectedInsideToInternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_relay, RTPortKindEnum.INTERNAL, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelayConnectedInsideToSAP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_relay, RTPortKindEnum.SAP, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelayConnectedInsideToSPP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_relay, RTPortKindEnum.SPP, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelayConnectedOutsideToRelay() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_relay, RTPortKindEnum.RELAY, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelayConnectedOutsideToExternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_relay, RTPortKindEnum.EXTERNAL, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelayConnectedOutsideToInternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_relay, RTPortKindEnum.INTERNAL, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelayConnectedOutsideToSAP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_relay, RTPortKindEnum.SAP, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromRelayConnectedOutsideToSPP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_relay, RTPortKindEnum.SPP, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternalConnectedInsideToRelay() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_internalBehavior, RTPortKindEnum.RELAY, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternalConnectedInsideToExternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_internalBehavior, RTPortKindEnum.EXTERNAL, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternalConnectedInsideToInternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_internalBehavior, RTPortKindEnum.INTERNAL, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternalConnectedInsideToSAP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_internalBehavior, RTPortKindEnum.SAP, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternalConnectedInsideToSPP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_internalBehavior, RTPortKindEnum.SPP, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternalConnectedOutsideToRelay() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_internalBehavior, RTPortKindEnum.RELAY, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternalConnectedOutsideToExternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_internalBehavior, RTPortKindEnum.EXTERNAL, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternalConnectedOutsideToInternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_internalBehavior, RTPortKindEnum.INTERNAL, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternalConnectedOutsideToSAP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_internalBehavior, RTPortKindEnum.SAP, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromInternalConnectedOutsideToSPP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_internalBehavior, RTPortKindEnum.SPP, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternalConnectedInsideToRelay() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_externalBehavior, RTPortKindEnum.RELAY, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternalConnectedInsideToExternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_externalBehavior, RTPortKindEnum.EXTERNAL, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternalConnectedInsideToInternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_externalBehavior, RTPortKindEnum.INTERNAL, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternalConnectedInsideToSAP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_externalBehavior, RTPortKindEnum.SAP, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternalConnectedInsideToSPP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule_externalBehavior, RTPortKindEnum.SPP, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternalConnectedOutsideToRelay() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_externalBehavior, RTPortKindEnum.RELAY, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternalConnectedOutsideToExternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_externalBehavior, RTPortKindEnum.EXTERNAL, true);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternalConnectedOutsideToInternal() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_externalBehavior, RTPortKindEnum.INTERNAL, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternalConnectedOutsideToSAP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_externalBehavior, RTPortKindEnum.SAP, false);
	}

	@Test
	public void testRTPortUtils_getChangeKindWithCheckFromExternalConnectedOutsideToSPP() throws Exception {
		testRTPortUtils_getChangeKindCommandWithCheck(connectedCapsule2_externalBehavior, RTPortKindEnum.SPP, false);
	}

	// ------------------------------------------------------------------------
	// RTPortUtils#isLegacySpp(Port)
	// ------------------------------------------------------------------------
	@Test
	public void testRTPortUtils_isLegacySpp_LegacySppTrue() throws Exception {
		Assert.assertTrue(RTPortUtils.isLegacySpp(capsule_legacySpp));
	}

	@Test
	public void testRTPortUtils_isLegacySpp_SppFalse() throws Exception {
		Assert.assertFalse(RTPortUtils.isLegacySpp(capsule_spp));
	}

	@Test
	public void testRTPortUtils_isLegacySpp_SapFalse() throws Exception {
		Assert.assertFalse(RTPortUtils.isLegacySpp(capsule_sap));
	}

	@Test
	public void testRTPortUtils_isLegacySpp_LegacySapFalse() throws Exception {
		Assert.assertFalse(RTPortUtils.isLegacySpp(capsule_legacySap));
	}

	@Test
	public void testRTPortUtils_isLegacySpp_ExternalFalse() throws Exception {
		Assert.assertFalse(RTPortUtils.isLegacySpp(capsule_externalBehavior));
	}

	@Test
	public void testRTPortUtils_isLegacySpp_InternalFalse() throws Exception {
		Assert.assertFalse(RTPortUtils.isLegacySpp(capsule_internalBehavior));
	}

	@Test
	public void testRTPortUtils_isLegacySpp_RelayFalse() throws Exception {
		Assert.assertFalse(RTPortUtils.isLegacySpp(capsule_relay));
	}

	// ------------------------------------------------------------------------
	// RTPortUtils#isLegacySap(Port)
	// ------------------------------------------------------------------------
	@Test
	public void testRTPortUtils_isLegacySap_LegacySppFalse() throws Exception {
		Assert.assertFalse(RTPortUtils.isLegacySap(capsule_legacySpp));
	}

	@Test
	public void testRTPortUtils_isLegacySap_SppFalse() throws Exception {
		Assert.assertFalse(RTPortUtils.isLegacySap(capsule_spp));
	}

	@Test
	public void testRTPortUtils_isLegacySap_SapFalse() throws Exception {
		Assert.assertFalse(RTPortUtils.isLegacySap(capsule_sap));
	}

	@Test
	public void testRTPortUtils_isLegacySap_LegacySapTrue() throws Exception {
		Assert.assertTrue(RTPortUtils.isLegacySap(capsule_legacySap));
	}

	@Test
	public void testRTPortUtils_isLegacySap_ExternalFalse() throws Exception {
		Assert.assertFalse(RTPortUtils.isLegacySap(capsule_externalBehavior));
	}

	@Test
	public void testRTPortUtils_isLegacySap_InternalFalse() throws Exception {
		Assert.assertFalse(RTPortUtils.isLegacySap(capsule_internalBehavior));
	}

	@Test
	public void testRTPortUtils_isLegacySap_RelayFalse() throws Exception {
		Assert.assertFalse(RTPortUtils.isLegacySap(capsule_relay));
	}

	// ------------------------------------------------------------------------
	// RTPortUtils#isConnected()
	// ------------------------------------------------------------------------
	@Test
	public void testRTPortUtils_isConnected_ExternalBehaviorConnected() throws Exception {
		Assert.assertTrue(RTPortUtils.isConnected(connectedCapsule_externalBehavior));
	}

	@Test
	public void testRTPortUtils_isConnected_InternalBehaviorConnected() throws Exception {
		Assert.assertTrue(RTPortUtils.isConnected(connectedCapsule_internalBehavior));
	}

	@Test
	public void testRTPortUtils_isConnected_RelayConnected() throws Exception {
		Assert.assertTrue(RTPortUtils.isConnected(connectedCapsule_relay));
	}

	@Test
	public void testRTPortUtils_isConnected_ExternalBehaviorNotConnected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnected(connectedCapsule_externalBehaviorNotConnected));
	}

	@Test
	public void testRTPortUtils_isConnected_InternalBehaviorNotConnected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnected(connectedCapsule_internalBehaviorNotConnected));
	}

	@Test
	public void testRTPortUtils_isConnected_RelayNotConnected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnected(connectedCapsule_relayNotConnected));
	}

	@Test
	public void testRTPortUtils_isConnected_ExternalBehavior2Connected() throws Exception {
		Assert.assertTrue(RTPortUtils.isConnected(connectedCapsule2_externalBehavior));
	}

	@Test
	public void testRTPortUtils_isConnected_InternalBehavior2Connected() throws Exception {
		Assert.assertTrue(RTPortUtils.isConnected(connectedCapsule2_internalBehavior));
	}

	@Test
	public void testRTPortUtils_isConnected_Relay2Connected() throws Exception {
		Assert.assertTrue(RTPortUtils.isConnected(connectedCapsule2_relay));
	}


	// ------------------------------------------------------------------------
	// RTPortUtils#isConnectedInside()
	// ------------------------------------------------------------------------
	@Test
	public void testRTPortUtils_isConnectedInside_ExternalBehaviorConnected() throws Exception {
		Assert.assertTrue(RTPortUtils.isConnectedInside(connectedCapsule_externalBehavior));
	}

	@Test
	public void testRTPortUtils_isConnectedInside_InternalBehaviorConnected() throws Exception {
		Assert.assertTrue(RTPortUtils.isConnectedInside(connectedCapsule_internalBehavior));
	}

	@Test
	public void testRTPortUtils_isConnectedInside_RelayConnected() throws Exception {
		Assert.assertTrue(RTPortUtils.isConnectedInside(connectedCapsule_relay));
	}

	@Test
	public void testRTPortUtils_isConnectedInside_ExternalBehaviorNotConnected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnectedInside(connectedCapsule_externalBehaviorNotConnected));
	}

	@Test
	public void testRTPortUtils_isConnectedInside_InternalBehaviorNotConnected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnectedInside(connectedCapsule_internalBehaviorNotConnected));
	}

	@Test
	public void testRTPortUtils_isConnectedInside_RelayNotConnected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnectedInside(connectedCapsule_relayNotConnected));
	}

	@Test
	public void testRTPortUtils_isConnectedInside_ExternalBehavior2Connected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnectedInside(connectedCapsule2_externalBehavior));
	}

	@Test
	public void testRTPortUtils_isConnectedInside_InternalBehavior2Connected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnectedInside(connectedCapsule2_internalBehavior));
	}

	@Test
	public void testRTPortUtils_isConnectedInside_Relay2Connected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnectedInside(connectedCapsule2_relay));
	}

	// ------------------------------------------------------------------------
	// RTPortUtils#isConnectedOutside()
	// ------------------------------------------------------------------------
	@Test
	public void testRTPortUtils_isConnectedOutside_ExternalBehaviorConnected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnectedOutside(connectedCapsule_externalBehavior));
	}

	@Test
	public void testRTPortUtils_isConnectedOutside_InternalBehaviorConnected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnectedOutside(connectedCapsule_internalBehavior));
	}

	@Test
	public void testRTPortUtils_isConnectedOutside_RelayConnected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnectedOutside(connectedCapsule_relay));
	}

	@Test
	public void testRTPortUtils_isConnectedOutside_ExternalBehaviorNotConnected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnectedOutside(connectedCapsule_externalBehaviorNotConnected));
	}

	@Test
	public void testRTPortUtils_isConnectedOutside_InternalBehaviorNotConnected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnectedOutside(connectedCapsule_internalBehaviorNotConnected));
	}

	@Test
	public void testRTPortUtils_isConnectedOutside_RelayNotConnected() throws Exception {
		Assert.assertFalse(RTPortUtils.isConnectedOutside(connectedCapsule_relayNotConnected));
	}

	@Test
	public void testRTPortUtils_isConnectedOutside_ExternalBehavior2Connected() throws Exception {
		Assert.assertTrue(RTPortUtils.isConnectedOutside(connectedCapsule2_externalBehavior));
	}

	@Test
	public void testRTPortUtils_isConnectedOutside_InternalBehavior2Connected() throws Exception {
		Assert.assertTrue(RTPortUtils.isConnectedOutside(connectedCapsule2_internalBehavior));
	}

	@Test
	public void testRTPortUtils_isConnectedOutside_Relay2Connected() throws Exception {
		Assert.assertTrue(RTPortUtils.isConnectedOutside(connectedCapsule2_relay));
	}


	// ------------------------------------------------------------------------
	// test methods
	// ------------------------------------------------------------------------

	protected void testRTPortUtils_getChangeKindCommand(Port portToModify, RTPortKindEnum newKind) {
		RTPortKindEnum oldKind = RTPortUtils.getKind(portToModify);

		Command command = RTPortUtils.getChangeKindCommand(portToModify, newKind);
		if (!command.canExecute()) {
			fail("Change command is not executable");
		}
		command.execute();
		checkPort(portToModify, newKind);

		if (!command.canUndo()) {
			fail("Change command is not undoable");
		}
		command.undo();
		checkPort(portToModify, oldKind);

		command.redo();
		checkPort(portToModify, newKind);

		command.undo();
		// check not dirty
	}

	protected void testRTPortUtils_getChangeKindCommandWithCheck(Port portToModify, RTPortKindEnum newKind, boolean isExecutable) {
		RTPortKindEnum oldKind = RTPortUtils.getKind(portToModify);

		Command command = RTPortUtils.getChangeKindCommand(portToModify, newKind, true);
		if (!isExecutable) { // not executable => do not run do/undo
			if (command.canExecute()) {
				fail("Command should not be executable");
			}
			return;
		}

		if (!command.canExecute()) {
			fail("Change command is not executable where it should");
		}

		command.execute();
		checkPort(portToModify, newKind);

		if (!command.canUndo()) {
			fail("Change command is not undoable");
		}
		command.undo();
		checkPort(portToModify, oldKind);

		command.redo();
		checkPort(portToModify, newKind);

		command.undo();
		// check not dirty
	}


	protected void checkPort(Port port, RTPortKindEnum kind) {
		switch (kind) {
		case EXTERNAL:
			assertTrue("Service: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isService(port));
			assertTrue("Wired: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isWired(port));
			assertTrue("Behavior: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isBehavior(port));
			assertFalse("Publish: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isPublish(port));
			assertEquals("Registration: error in RTPortUtils#getChangeKindCommand(Port)", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(port));
			assertEquals("RegistrationOverride: error in RTPortUtils#getChangeKindCommand(Port)", "", RTPortUtils.getRegistrationOverride(port));
			assertEquals("Visibility: error in RTPortUtils#getChangeKindCommand(Port)", VisibilityKind.PUBLIC_LITERAL, port.getVisibility());
			break;
		case INTERNAL:
			assertFalse("Service: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isService(port));
			assertTrue("Wired: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isWired(port));
			assertTrue("Behavior: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isBehavior(port));
			assertFalse("Publish: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isPublish(port));
			assertEquals("Registration: error in RTPortUtils#getChangeKindCommand(Port)", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(port));
			assertEquals("RegistrationOverride: error in RTPortUtils#getChangeKindCommand(Port)", "", RTPortUtils.getRegistrationOverride(port));
			assertEquals("Visibility: error in RTPortUtils#getChangeKindCommand(Port)", VisibilityKind.PROTECTED_LITERAL, port.getVisibility());
			break;
		case RELAY:
			assertTrue("Service: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isService(port));
			assertTrue("Wired: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isWired(port));
			assertFalse("Behavior: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isBehavior(port));
			assertFalse("Publish: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isPublish(port));
			assertEquals("Registration: error in RTPortUtils#getChangeKindCommand(Port)", PortRegistrationType.AUTOMATIC, RTPortUtils.getRegistration(port));
			assertEquals("RegistrationOverride: error in RTPortUtils#getChangeKindCommand(Port)", "", RTPortUtils.getRegistrationOverride(port));
			assertEquals("Visibility: error in RTPortUtils#getChangeKindCommand(Port)", VisibilityKind.PUBLIC_LITERAL, port.getVisibility());
			break;
		case SPP:
			// assertTrue("Service: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isService(port)); // legacy
			assertFalse("Wired: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isWired(port));
			assertTrue("Behavior: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isBehavior(port));
			assertTrue("Publish: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isPublish(port));
			assertEquals("Visibility: error in RTPortUtils#getChangeKindCommand(Port)", VisibilityKind.PUBLIC_LITERAL, port.getVisibility());
			break;
		case SAP:
			// assertTrue("Service: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isService(port));// legacy
			assertFalse("Wired: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isWired(port));
			assertTrue("Behavior: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isBehavior(port));
			assertFalse("Publish: error in RTPortUtils#getChangeKindCommand(Port)", RTPortUtils.isPublish(port));
			assertEquals("Visibility: error in RTPortUtils#getChangeKindCommand(Port)", VisibilityKind.PROTECTED_LITERAL, port.getVisibility());
			break;

		default:
			break;
		}
	}

}
