/*****************************************************************************
 * Copyright (c) 2016 Christian W. Damus, CEA LIST, and others.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Christian W. Damus - Initial API and implementation
 *   CEA LIST - bug 489624
 *   Christian W. Damus - bug 496464
 *   
 *****************************************************************************/

package org.eclipse.papyrusrt.umlrt.tooling.diagram.statemachine.canonical.tests;

import static org.hamcrest.CoreMatchers.notNullValue;
import static org.hamcrest.CoreMatchers.nullValue;
import static org.hamcrest.MatcherAssert.assertThat;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.eclipse.gmf.runtime.common.core.command.ICommand;
import org.eclipse.gmf.runtime.emf.type.core.ElementTypeRegistry;
import org.eclipse.gmf.runtime.emf.type.core.IHintedType;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateElementRequest;
import org.eclipse.papyrus.infra.services.edit.service.ElementEditServiceUtils;
import org.eclipse.papyrus.infra.services.edit.service.IElementEditService;
import org.eclipse.papyrus.junit.utils.rules.PluginResource;
import org.eclipse.papyrusrt.junit.rules.UMLRTViewpointRule;
import org.eclipse.papyrusrt.umlrt.tooling.diagram.common.canonical.tests.AbstractCanonicalTest;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Pseudostate;
import org.eclipse.uml2.uml.Region;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.Transition;
import org.eclipse.uml2.uml.UMLPackage;
import org.junit.Before;
import org.junit.ClassRule;
import org.junit.Test;
import org.junit.rules.TestRule;

@PluginResource("resource/statemachine/model.di")
public class StateMachineDiagramTest extends AbstractCanonicalTest {
	@ClassRule
	public static final TestRule VIEWPOINT_RULE = new UMLRTViewpointRule();

	private Class capsule1;
	private StateMachine capsule1_stateMachine;
	private Region capsule1_stateMachine_region;
	private Pseudostate capsule1_stateMachine_initialState;
	private State capsule1_stateMachine_state1;
	private Transition capsule1_stateMachine_initialTransition;

	/**
	 * Verify that when a port is changed to a non-service port, it disappears
	 * from the capsule parts typed by its featuring capsules.
	 */
	@NeedsUIEvents
	@Test
	public void createEntryState() {
		assertThat("Entry action found before creating it!", capsule1_stateMachine_state1.getEntry(), nullValue());
		execute(new CreateEntryActionHelper().getCreateEntryCommand(capsule1_stateMachine_state1));

		assertThat("Impossible to find Entry action", capsule1_stateMachine_state1.getEntry(), notNullValue());
	}

	//
	// Test framework
	//

	@Before
	public void getFixtures() {
		capsule1 = (Class) editor.getModel().getOwnedType("Capsule1");
		capsule1_stateMachine = (StateMachine) capsule1.getClassifierBehavior();
		capsule1_stateMachine_region = capsule1_stateMachine.getRegion("Region", false, false);
		capsule1_stateMachine_initialState = (Pseudostate) capsule1_stateMachine_region.getSubvertex(null, false, UMLPackage.eINSTANCE.getPseudostate(), false);
		capsule1_stateMachine_state1 = (State) capsule1_stateMachine_region.getSubvertex("State1", true, UMLPackage.eINSTANCE.getState(), false);
		capsule1_stateMachine_initialTransition = capsule1_stateMachine_initialState.getOutgoing("Initial");
	}

	private static class CreateEntryActionHelper {
		ICommand getCreateEntryCommand(State state) {
			IHintedType opaqueBehaviorElementType = (IHintedType) ElementTypeRegistry.getInstance().getType("org.eclipse.papyrus.uml.OpaqueBehavior");
			assertThat("Impossible to find element type for OpaqueBehavior", opaqueBehaviorElementType, notNullValue());
			CreateElementRequest request = new CreateElementRequest(state, opaqueBehaviorElementType, UMLPackage.eINSTANCE.getState_Entry());
			final EObject target = ElementEditServiceUtils.getTargetFromContext(TransactionUtil.getEditingDomain(state), state, request);

			IElementEditService provider = ElementEditServiceUtils.getCommandProvider(target);
			if (provider == null) {
				return org.eclipse.gmf.runtime.common.core.command.UnexecutableCommand.INSTANCE;
			}
			return provider.getEditCommand(new CreateElementRequest(TransactionUtil.getEditingDomain(state), target, opaqueBehaviorElementType, UMLPackage.eINSTANCE.getState_Entry()));
		}
	}

}
